package cn.com.qmth.examcloud.exchange.outer.api.provider;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.HttpClientUtil;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.UpdatePasswordReq;
import cn.com.qmth.examcloud.core.basic.api.request.UpdateStudentStatusReq;
import cn.com.qmth.examcloud.core.basic.api.response.SaveStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.StudentOuterService;
import cn.com.qmth.examcloud.exchange.outer.api.bean.Student4BatchBean;
import cn.com.qmth.examcloud.exchange.outer.api.bean.StudentStatus4BatchBean;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterBatchSaveStudentReq;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterSaveStudentReq;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterUpdatePasswordReq;
import cn.com.qmth.examcloud.exchange.outer.api.request.OuterUpdateStudentStatusReq;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterBatchSaveStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterSaveStudentResp;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterUpdatePasswordResp;
import cn.com.qmth.examcloud.exchange.outer.api.response.OuterUpdateStudentStatusResp;
import cn.com.qmth.examcloud.exchange.outer.service.FaceService;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.StatusResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年6月29日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Api(tags = "学生相关接口")
@RestController
@RequestMapping("${$rmp.cloud.exchange.outer}/student")
public class StudentOuterServiceProvider extends ControllerSupport implements StudentOuterService {

	private static final long serialVersionUID = -8252740695302763716L;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	private FaceService faceService;

	@Autowired
	SystemProperties systemConfig;

	@ApiOperation(value = "保存学生信息")
	@ApiResponses({@ApiResponse(code = 200, message = "成功", response = OuterSaveStudentResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("saveStudent")
	@Override
	public OuterSaveStudentResp saveStudent(
			@RequestBody @ApiParam(required = true) OuterSaveStudentReq req) {
		trim(req);
		Long rootOrgId = req.getRootOrgId();
		if (!getEnterpriseRootOrgId().equals(rootOrgId)) {
			throw new StatusException("1000001", "rootOrgId is wrong");
		}

		SaveStudentReq request = new SaveStudentReq();
		request.setEnable(req.getEnable());
		request.setIdentityNumber(req.getIdentityNumber());
		request.setName(req.getName());
		request.setOrgCode(req.getOrgCode());
		request.setOrgId(req.getOrgId());
		request.setOrgName(req.getOrgName());
		request.setPhoneNumber(req.getPhoneNumber());
		request.setRemark(req.getRemark());
		request.setRootOrgId(req.getRootOrgId());
		if (StringUtils.isNotBlank(req.getStudentCode())) {
			List<String> studentCodeList = Lists.newArrayList();
			studentCodeList.add(req.getStudentCode());
			request.setStudentCodeList(studentCodeList);
		}

		// 保存学生
		SaveStudentResp response = studentCloudService.saveStudent(request);
		// 处理照片
		processPhotoUrl(rootOrgId, req.getIdentityNumber(), req.getPhotoUrl(), req.getOperator());

		OuterSaveStudentResp resp = new OuterSaveStudentResp();
		resp.setOrgId(response.getOrgId());
		resp.setOrgName(response.getOrgName());
		resp.setRootOrgId(response.getRootOrgId());
		resp.setStudentId(response.getStudentId());
		return resp;
	}

	/**
	 * 处理照片
	 *
	 * @author WANGWEI
	 * @param rootOrgId
	 * @param identityNumber
	 * @param photoUrl
	 * @param operator
	 */
	private void processPhotoUrl(Long rootOrgId, String identityNumber, String photoUrl,
			String operator) {

		if (StringUtils.isNotBlank(photoUrl)) {
			int lastIndexOf = photoUrl.lastIndexOf(".");
			if (0 > lastIndexOf) {
				throw new StatusException("100002", "photoPath is not end with photo file suffix.");
			}

			String fileSuffix = photoUrl.substring(lastIndexOf);
			String fileName = System.currentTimeMillis() + fileSuffix;
			File temp = new File(systemConfig.getTempDataDir() + File.separator + "student_photo"
					+ File.separator + fileName);

			try {
				byte[] bs = HttpClientUtil.get(photoUrl);

				FileUtils.writeByteArrayToFile(temp, bs);
			} catch (IOException e) {
				try {
					FileUtils.forceDelete(temp);
				} catch (IOException ex) {
					// ignore
				}
				throw new StatusException("100003", "文件读写失败", e);
			}

			try {
				faceService.processFace(rootOrgId, identityNumber, fileSuffix, temp, operator);
			} finally {
				try {
					FileUtils.forceDelete(temp);
				} catch (IOException ex) {
					// ignore
				}
			}
		}
	}

	@ApiOperation(value = "批量保存学生信息")
	@ApiResponses({
			@ApiResponse(code = 200, message = "成功", response = OuterBatchSaveStudentResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("batchSaveStudent")
	@Override
	public OuterBatchSaveStudentResp batchSaveStudent(
			@RequestBody @ApiParam(required = true) OuterBatchSaveStudentReq req) {

		if (!getEnterpriseRootOrgId().equals(req.getRootOrgId())) {
			throw new StatusException("1000001", "rootOrgId is wrong");
		}

		List<Student4BatchBean> studentList = req.getStudentList();

		List<StudentStatus4BatchBean> successList = Lists.newArrayList();
		List<StudentStatus4BatchBean> failureList = Lists.newArrayList();

		for (Student4BatchBean cur : studentList) {
			SaveStudentReq ssReq = new SaveStudentReq();
			ssReq.setEnable(cur.getEnable());
			ssReq.setIdentityNumber(cur.getIdentityNumber());
			ssReq.setName(cur.getName());
			ssReq.setOrgCode(cur.getOrgCode());
			ssReq.setOrgId(cur.getOrgId());
			ssReq.setOrgName(cur.getOrgName());
			ssReq.setPhoneNumber(cur.getPhoneNumber());
			ssReq.setRemark(cur.getRemark());
			ssReq.setRootOrgId(cur.getRootOrgId());
			if (StringUtils.isNotBlank(cur.getStudentCode())) {
				List<String> studentCodeList = Lists.newArrayList();
				studentCodeList.add(cur.getStudentCode());
				ssReq.setStudentCodeList(studentCodeList);
			}

			try {

				// 保存学生
				SaveStudentResp ssResp = studentCloudService.saveStudent(ssReq);
				// 处理照片
				processPhotoUrl(req.getRootOrgId(), cur.getIdentityNumber(), cur.getPhotoUrl(),
						cur.getOperator());

				StudentStatus4BatchBean studentStatus4BatchBeanSuccess = new StudentStatus4BatchBean();
				studentStatus4BatchBeanSuccess.setStudentId(ssResp.getStudentId());
				studentStatus4BatchBeanSuccess.setIdentityNumber(cur.getIdentityNumber());
				studentStatus4BatchBeanSuccess.setName(cur.getName());
				studentStatus4BatchBeanSuccess.setStudentCode(cur.getStudentCode());
				successList.add(studentStatus4BatchBeanSuccess);
			} catch (Exception e) {
				StudentStatus4BatchBean studentStatus4BatchBeanfailure = new StudentStatus4BatchBean();
				studentStatus4BatchBeanfailure.setIdentityNumber(cur.getIdentityNumber());
				studentStatus4BatchBeanfailure.setName(cur.getName());
				studentStatus4BatchBeanfailure.setStudentCode(cur.getStudentCode());
				failureList.add(studentStatus4BatchBeanfailure);
			}
		}
		OuterBatchSaveStudentResp resq = new OuterBatchSaveStudentResp();
		resq.setSuccessList(successList);
		resq.setFailureList(failureList);
		return resq;
	}

	@ApiOperation(value = "更新学生密码")
	@ApiResponses({
			@ApiResponse(code = 200, message = "成功", response = OuterUpdatePasswordResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("updatePassword")
	@Override
	public OuterUpdatePasswordResp updatePassword(
			@RequestBody OuterUpdatePasswordReq req) {

		if (!getEnterpriseRootOrgId().equals(req.getRootOrgId())) {
			throw new StatusException("1000001", "rootOrgId is wrong");
		}

		UpdatePasswordReq upReq = new UpdatePasswordReq();
		upReq.setIdentityNumber(req.getIdentityNumber());
		upReq.setRootOrgId(req.getRootOrgId());
		upReq.setStudentId(req.getStudentId());
		upReq.setPassword(req.getPassword());
		studentCloudService.updatePassword(upReq);

		OuterUpdatePasswordResp resp = new OuterUpdatePasswordResp();
		return resp;
	}

	@ApiOperation(value = "更新学生状态")
	@ApiResponses({
			@ApiResponse(code = 200, message = "成功", response = OuterUpdateStudentStatusResp.class),
			@ApiResponse(code = 500, message = "系统异常(异常信息见响应体)", response = StatusResponse.class)})
	@PostMapping("updateStudentStatus")
	@Override
	public OuterUpdateStudentStatusResp updateStudentStatus(
			@RequestBody OuterUpdateStudentStatusReq req) {

		if (!getEnterpriseRootOrgId().equals(req.getRootOrgId())) {
			throw new StatusException("1000001", "rootOrgId is wrong");
		}

		UpdateStudentStatusReq ussReq = new UpdateStudentStatusReq();
		ussReq.setIdentityNumber(req.getIdentityNumber());
		ussReq.setRootOrgId(req.getRootOrgId());
		ussReq.setStudentId(req.getStudentId());
		ussReq.setEnable(req.getEnable());
		studentCloudService.updateStudentStatus(ussReq);

		OuterUpdateStudentStatusResp resp = new OuterUpdateStudentStatusResp();
		return resp;
	}

}
